#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rgrid.h"

extern void *scalloc(size_t nmemb, size_t size);

EightGrid::EightGrid(int w, int l, double cls, double **s) :
	RGrid2D(w, l, cls, s)
{
   int i, j;
   
   D1Connected = (int **)scalloc(Width - 1, sizeof(int *));
   D2Connected = (int **)scalloc(Width - 1, sizeof(int *));
   D1Curr = (double **)scalloc(Width - 1, sizeof(double *));
   D2Curr = (double **)scalloc(Width - 1, sizeof(double *));
   D1Diff = (double **)scalloc(Width - 1, sizeof(double *));
   D2Diff = (double **)scalloc(Width - 1, sizeof(double *));

   for (i = 0; i < Width - 1; i++)
     {
	D1Connected[i] = (int *)scalloc(Length - 1, sizeof(int));
	for (j = 0; j < Length - 1; j++)
	  D1Connected[i][j] = 1;
	D1Curr[i] = (double *)scalloc(Length - 1, sizeof(double));
	D1Diff[i] = (double *)scalloc(Length - 1, sizeof(double));
	D2Connected[i] = (int *)scalloc(Length - 1, sizeof(int));
	for (j = 0; j < Length - 1; j++)
	  D2Connected[i][j] = 1;
	D2Curr[i] = (double *)scalloc(Length - 1, sizeof(double));
	D2Diff[i] = (double *)scalloc(Length - 1, sizeof(double));
     }
}

EightGrid::~EightGrid() :
	~RGrid2D()
{
   int i;

   for (i = 0; i < Width - 1; i++)
     {
	free(D1Connected[i]);
	free(D1Curr[i]);
	free(D1Diff[i]);
	free(D2Connected[i]);
	free(D2Curr[i]);
	free(D2Diff[i]);
     }
   free(D1Connected);
   free(D2Connected);
   free(D1Curr);
   free(D2Curr);
   free(D1Diff);
   free(D2Diff);
}

void EightGrid::IterateD1(int i, int j, float Increment)
{
   double newcurr;
   
   D1Diff[i][j] = Nodes[i][j] - Nodes[i + 1][j + 1];
   if (D1Connected[i][j])
     newcurr = D1Diff[i][j];
   else
     newcurr = 0.0;
   D1Curr[i][j] = D1Curr[i][j] * (1.0 - Increment) + newcurr * Increment;
}

void EightGrid::IterateD2(int i, int j, float Increment)
{
   double newcurr;
   
   D2Diff[i][j] = Nodes[i + 1][j] - Nodes[i][j + 1];
   if (D2Connected[i][j])
     newcurr = D2Diff[i][j];
   else
     newcurr = 0.0;
   D2Curr[i][j] = D2Curr[i][j] * (1.0 - Increment) + newcurr * Increment;
}

void EightGrid::CalcNode(int i, int j, float Increment)
{
   double curr = 0.0;
   double newnode;
   double change = Nodes[i][j];

   if (i != 0)
     curr += HCurr[i - 1][j];
   if (i != Width - 1)
     curr -= HCurr[i][j];
   if (j != 0)
     curr += VCurr[i][j - 1];
   if (j != Length - 1)
     curr -= VCurr[i][j];
   if ((i != 0) && (j != 0))
     curr += D1Curr[i - 1][j - 1];
   if ((i != Width - 1) && (j != Length - 1))
     curr -= D1Curr[i][j];
   if ((i != Width - 1) && (j != 0))
     curr += D2Curr[i][j - 1];
   if ((i != 0) && (j != Length - 1))
     curr -= D2Curr[i - 1][j];
   newnode = Sources[i][j] + curr / CLengthSqu;
   Nodes[i][j] = Nodes[i][j] * (1.0 - Increment) + newnode * Increment;
   change -= Nodes[i][j];
   if (fabs(change) >= LargestChange)
     LargestChange = fabs(change);
}

double EightGrid::Iterate(float Increment)
{
   int i, j;

   LargestChange = 0.0;
   for (i = 0; i < Width - 1; i++)
     {
	for (j = 0; j < Length; j++)
	  IterateH(i, j, Increment);
     }
   for (i = 0; i < Width; i++)
     {
	for (j = 0; j < Length - 1; j++)
	  IterateV(i, j, Increment);
     }
   for (i = 0; i < Width - 1; i++)
     {
	for (j = 0; j < Lenght - 1; j++)
	  {
	     IterateD1(i, j, Increment);
	     IterateD2(i, j, Increment);
	  }
     }
   for (i = 0; i < Width; i++)
     {
	for (j = 0; j < Length; j++)
	  CalcNode(i, j, Increment);
     }
   return LargestChange;
}

void RGrid2D::PrintHCurrTable()
{
   int i, j;
   
   for (j = 0; j < Length; j++)
     {
	for (i = 0; i < Width - 1; i++)
	  printf("%.2f ", HCurr[i][j]);
	printf("\n");
     }
}

void RGrid2D::PrintVCurrTable()
{
   int i, j;
   
   for (j = 0; j < Length - 1; j++)
     {
	for (i = 0; i < Width; i++)
	  printf("%.2f ", VCurr[i][j]);
	printf("\n");
     }
}

void RGrid2D::PrintSegmentation()
{
   int i, j;
   
   for (j = 0; j < Length - 1; j++)
     {
	for (i = 0; i < Width - 1; i++)
	  printf(HConnected[i][j] ? " ---":"    ");
	printf("\n");
	for (i = 0; i < Width - 1; i++)
	  {
	     printf(VConnected[i][j] ? "|":" ");
	     printf("   ");
	  }
	printf(VConnected[Width - 1][j] ? "|\n":" \n");
     }
   for (i = 0; i < Width - 1; i++)
     printf(HConnected[i][Length - 1] ? " ---":"    ");
   printf("\n");
}

void RGrid2D::PrintHeader(FILE *f, const char *hfile)
{
   FILE *header;
   char c;

   fprintf(f, "%%!PS-Adobe-2.0\n");
   fprintf(f, "%%%%BoundingBox: 0 0 %d %d\n\n", Width, Length);
   header = fopen(hfile, "r");
   while ((c = getc(header)) != EOF)
     putc(c, f);
   fclose(header);
}

void RGrid2D::PrintNodes(FILE *f)
{
   int i, j;

   for (i = 0; i < Width; i++)
     {
	for (j = 0; j < Length; j++)
	  fprintf(f, "%3d %3d %.3f pixel\n", i, j, (Nodes[i][j] - 1.5) / 2.0);
     }
}

void RGrid2D::PrintPostscript(const char *psfile)
{
   FILE *f = fopen(psfile, "w");

   PrintHeader(f, "header.ps");
   PrintNodes(f);
   fclose(f);
}

void RGrid2D::PrintEdgesPostscript(const char *psfile)
{
   FILE *f = fopen(psfile, "w");
   int i, j;

   PrintHeader(f, "header.ps");
   PrintNodes(f);
   for (i = 0; i < Width; i++)
     {
	for (j = 0; j < Length - 1; j++)
	  {
	     if (VConnected[i][j] == 0)
	       fprintf(f, "%d %d %d %d dotline\n", i, j + 1, i + 1, j + 1);
	  }
     }
   for (i = 0; i < Width - 1; i++)
     {
	for (j = 0; j < Length; j++)
	  {
	     if (HConnected[i][j] == 0)
	       fprintf(f, "%d %d %d %d dotline\n", i + 1, j, i + 1, j + 1);
	  }
     }
   fclose(f);
}

