#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mygrid.h"

#define MAXITERATIONS 100

#define INCREMENT 0.1
#define TOLERANCE 0.0001

#define WIDTH 8
#define LENGTH 1

#define NOISE 0.0

void *scalloc(size_t nmemb, size_t size)
{
   void *retval = calloc(nmemb, size);
   if (retval == NULL)
     {
	printf("Sorry, couldn't allocate %d * %d bytes\n", (int)nmemb, (int)size);
	exit(1);
     }
   return retval;
}

/*#define MYGRID*/

int main()
{
   int i, j, iterations = 0;
   double **Sources = (double **)scalloc(WIDTH, sizeof(double *));
#ifdef MYGRID
   MyGrid *RG;
#else
   RFGrid2D *RG;
#endif
   double LargestChange;

   srand(1);
#ifdef MYGRID
   RG = new MyGrid(WIDTH, LENGTH, 0.75, 1.5, 0.2, 10.0, Sources);
#else
   RG = new RFGrid2D(WIDTH, LENGTH, 0.75, 0.45, Sources);
#endif
   for (i = 0; i < WIDTH; i++)
     {
	Sources[i] = (double *)scalloc(LENGTH, sizeof(double));
	for (j = 0; j < LENGTH; j++)
	  {
	     if (i < 3)
	       Sources[i][j] = 2.0;
	     else
	       Sources[i][j] = 3.0;
	  }
     }
   RG->InitialiseNodes();
   while ((LargestChange = RG->Iterate(INCREMENT)) >= TOLERANCE);
   printf("%.2f\t", Sources[3][0]);
   RG->PrintNodeTable();
   for (i = 0; i < 5; i++)
     {
	Sources[3][0] -= 0.2;
	printf("%.2f\t", Sources[3][0]);
	while ((LargestChange = RG->Iterate(INCREMENT)) >= TOLERANCE);
	RG->PrintNodeTable();
     }
   for (i = 0; i < 5; i++)
     {
	Sources[4][0] -= 0.2;
	printf("%.2f\t", Sources[4][0]);
	while ((LargestChange = RG->Iterate(INCREMENT)) >= TOLERANCE);
	RG->PrintNodeTable();
     }
   
   fflush(stdout);
   return 0;
}

