#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "mygrid.h"

#define MAXITERATIONS 100

#define INCREMENT 0.1
#define TOLERANCE 0.0001

#define WIDTH 8
#define LENGTH 16

#define NOISE 0.4

void *scalloc(size_t nmemb, size_t size)
{
   void *retval = calloc(nmemb, size);
   if (retval == NULL)
     {
	printf("Sorry, couldn't allocate %d * %d bytes\n", (int)nmemb, (int)size);
	exit(1);
     }
   return retval;
}

float ImageInput(int i, int j)
{
   float factor1, factor2;

   if (i < 4)
     factor1 = (float)i / 3.0;
   else
     factor1 = ((float)i - 7.0) / 3.0;
   factor2 = j;
   factor2 = 1.0;

   return 2.5 + 0.5 * factor1 * factor2;
}

void AddNoise(double **s)
{
   int i, j;
   
   for (i = 0; i < WIDTH; i++)
     {
	for (j = 0; j < LENGTH; j++)
	  s[i][j] += NOISE * (2 * (float)(rand()) / RAND_MAX - 1.0);
     }
}

/*#define MYGRID*/

int main()
{
   int i, j, iterations = 0;
   double **Sources = (double **)scalloc(WIDTH, sizeof(double *));
#ifdef MYGRID
   MyGrid *RG;
#else
   RFGrid2D *RG;
#endif
   double LargestChange;

   srand(1);
#ifdef MYGRID
   RG = new MyGrid(WIDTH, LENGTH, 0.75, 1.5, 0.2, 10.0, Sources);
#else
   RG = new RFGrid2D(WIDTH, LENGTH, 0.75, 0.45, Sources);
#endif
   for (i = 0; i < WIDTH; i++)
     {
	Sources[i] = (double *)scalloc(LENGTH, sizeof(double));
	for (j = 0; j < LENGTH; j++)
	  {
	     Sources[i][j] = ImageInput(i, j);
	  }
     }
   RG->InitialiseNodes();
   RG->PrintPostscript("signal.ps");
   AddNoise(Sources);
   RG->InitialiseNodes();
   printf("\nNode table:\n");
   RG->PrintNodeTable();
   RG->PrintPostscript("noisy.ps");
   while ((LargestChange = RG->Iterate(INCREMENT)) >= TOLERANCE)
     {
	iterations++;
	printf("Iteration %d, largest change %f\n", iterations, LargestChange);
	if (iterations >= MAXITERATIONS)
	  {
	     printf("***** Reached Maximum Iterations *****\n");
	     break;
	  }
     }
   printf("\nNode table:\n");
   RG->PrintNodeTable();
   printf("\nSegmentation:\n");
   RG->PrintSegmentation();
#ifdef MYGRID
   printf("\nHorizontal Nodes:\n");
   RG->HGrid->PrintNodeTable();
   printf("\nVertical Nodes:\n");
   RG->VGrid->PrintNodeTable();
#endif
   fflush(stdout);
   RG->PrintPostscript("out.ps");
   RG->PrintEdgesPostscript("line.ps");
   return 0;
}

