#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define NOISE 0.1
#define WIDTH 8
#define LENGTH 16
#define FUSE
/*#define INTER_RFUSE*/
/*#define THREEDIGITNODES*/
/*#define USENODESETS*/

float input[WIDTH * LENGTH];

float ImageInput(int i, int j)
{
   float factor1, factor2;
   if (i <= 4)
     factor1 = ((float)i - 1.0) / 3.0;
   else
     factor1 = ((float)i - 8.0) / 3.0;
   factor2 = ((float)j - 1.0) / 15.0;
   return 2.5 + 0.5 * factor1 * factor2;
}

void CreateInput()
{
   int i, j;
   
   for (i = 0; i < WIDTH; i++)
     {
	for (j = 0; j < LENGTH; j++)
	  {
	     input[j * WIDTH + i] = ImageInput(i + 1, j + 1);
	  }
     }
}

#ifdef THREEDIGITNODES

void Source(int i, int j, float voltage)
{
   float noise = NOISE * (2.0 * ((float)rand() / (float)RAND_MAX) - 1);
   printf("VS%02d%02d\t1%02d\t0\t%f\n", i, j, i + j * WIDTH, voltage);
   printf("VN%02d%02d\t7%02d\t1%02d\t%f\n", i, j, i + j * WIDTH, i + j * WIDTH, noise);
   printf("R%02d%02d\t7%02d\t2%02d\t200k\n",
	  i, j, i + j * WIDTH, i + j * WIDTH);
}

void HorFuse(int i, int j)
{
#ifdef FUSE
   printf("XA%02d%02d\t1\t2%02d\t2%02d\t4 5 6\t",
	  i, j, i + j * WIDTH, i+1 + j * WIDTH);
   printf("3%02d%02d\t4%02d%02d\tRFNEW\n", i, j, i, j);
#else
   printf("RA%02d%02d\t2%02d\t2%02d\t150k\n", i, j, i + j * WIDTH, i+1 + j * WIDTH);
#endif
}

void HorFuseHorRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XB%02d%02d\t1\t3%02d\t3%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i+1 + j * WIDTH);
   printf("XC%02d%02d\t1\t4%02d\t4%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i+1 + j * WIDTH);
#endif 
}

void HorFuseVerRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XD%02d%02d\t1\t3%02d\t3%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i + (j+1) * WIDTH);
   printf("XE%02d%02d\t1\t4%02d\t4%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i + (j+1) * WIDTH);
#endif
}

void VerFuse(int i, int j)
{
#ifdef FUSE
   printf("XF%02d%02d\t1\t2%02d\t2%02d\t4 5 6\t",
	  i, j, i + j * WIDTH, i + (j+1) * WIDTH);
   printf("5%02d\t6%02d\tRFNEW\n", i + j * WIDTH, i + j * WIDTH);
#else
   printf("RF%02d%02d\t2%02d\t2%02d\t150k\n", i, j,
	  i + j * WIDTH, i + (j+1) * WIDTH);
#endif
}

void VerFuseHorRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XG%02d%02d\t1\t5%02d\t5%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i+1 + j * WIDTH);
   printf("XH%02d%02d\t1\t6%02d\t6%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i+1 + j * WIDTH);
#endif
}

void VerFuseVerRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XI%02d%02d\t1\t5%02d\t5%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i + (j+1) * WIDTH);
   printf("XJ%02d%02d\t1\t6%02d\t6%02d\t7 8 NEGRES\n",
	  i, j, i + j * WIDTH, i + (j+1) * WIDTH);
#endif
}

#else /* #ifdef THREEDIGITNODES */

void Source(int i, int j, float voltage)
{
   float noise = NOISE * (2.0 * ((float)rand() / (float)RAND_MAX) - 1);
   printf("VS%02d%02d\t1%02d%02d\t0\t%f\n", i, j, i, j, voltage);
   printf("VN%02d%02d\t7%02d%02d\t1%02d%02d\t%f\n", i, j, i, j, i, j, noise);
   printf("R%02d%02d\t7%02d%02d\t2%02d%02d\t200k\n",
	  i, j, i, j, i, j);
#ifdef USENODESETS
   printf(".NODESET V(1%02d%02d) = %f\n", i, j, voltage);
   printf(".NODESET V(7%02d%02d) = %f\n", i, j, voltage + noise);
   printf(".NODESET V(2%02d%02d) = %f\n", i, j, voltage + noise);
#endif
}

void HorFuse(int i, int j)
{
#ifdef FUSE
   printf("XA%02d%02d\t1\t2%02d%02d\t2%02d%02d\t4 5 6\t",
	  i, j, i, j, i+1, j);
   printf("3%02d%02d\t4%02d%02d\tRFNEW\n", i, j, i, j);
#else
   printf("RA%02d%02d\t2%02d%02d\t2%02d%02d\t150k\n", i, j, i, j, i+1, j);
#endif
}

void HorFuseHorRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XB%02d%02d\t1\t3%02d%02d\t3%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i+1, j);
   printf("XC%02d%02d\t1\t4%02d%02d\t4%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i+1, j);
#endif 
}

void HorFuseVerRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XD%02d%02d\t1\t3%02d%02d\t3%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i, j+1);
   printf("XE%02d%02d\t1\t4%02d%02d\t4%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i, j+1);
#endif
}

void VerFuse(int i, int j)
{
#ifdef FUSE
   printf("XF%02d%02d\t1\t2%02d%02d\t2%02d%02d\t4 5 6\t",
	  i, j, i, j, i, j+1);
   printf("5%02d%02d\t6%02d%02d\tRFNEW\n", i, j, i, j);
#else
   printf("RF%02d%02d\t2%02d%02d\t2%02d%02d\t150k\n", i, j, i, j, i, j+1);
#endif
}

void VerFuseHorRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XG%02d%02d\t1\t5%02d%02d\t5%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i+1, j);
   printf("XH%02d%02d\t1\t6%02d%02d\t6%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i+1, j);
#endif
}

void VerFuseVerRes(int i, int j)
{
#ifdef INTER_RFUSE
   printf("XI%02d%02d\t1\t5%02d%02d\t5%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i, j+1);
   printf("XJ%02d%02d\t1\t6%02d%02d\t6%02d%02d\t7 8 NEGRES\n",
	  i, j, i, j, i, j+1);
#endif
}

#endif /* #ifdef THREEDIGITNODES */

int main()
{
   FILE *f;
   int i, j;
   char c;

   srand(1);
   f = fopen("header.cir", "r");
   while ((c = getc(f)) != EOF)
     {
	putchar(c);
     }
   fclose(f);
   printf("*NETWORKWIDTH %d\n", WIDTH);
   printf("*NETWORKLENGTH %d\n\n", LENGTH);
   CreateInput();
   printf("* ARRAY VALUES\n");
   for (j = 0; j < LENGTH; j++)
     {
	printf("* ");
	for (i = 0; i < WIDTH; i++)
	  printf("%.2f ", input[j * WIDTH + i]);
	printf("\n");
     }
   printf("\n");
   for (i = 1; i < WIDTH - 1; i++)
     {
	for (j = 1; j < LENGTH - 1; j++)
	  {
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	     HorFuseHorRes(i, j);
	     HorFuseVerRes(i, j);
	     VerFuse(i, j);
	     VerFuseHorRes(i, j);
	     VerFuseVerRes(i, j);
	  }
	if (LENGTH > 1)
	  {
	     j = LENGTH - 1;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	     HorFuseHorRes(i, j);
	     HorFuseVerRes(i, j);
	     VerFuse(i, j);
	     VerFuseHorRes(i, j);
	  }
	if (LENGTH >= 1)
	  {
	     j = LENGTH;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	     HorFuseHorRes(i, j);
	  }
     }
   if (WIDTH > 1)
     {
	i = WIDTH - 1;
	for (j = 1; j < LENGTH - 1; j++)
	  {
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	     HorFuseVerRes(i, j);
	     VerFuse(i, j);
	     VerFuseHorRes(i, j);
	     VerFuseVerRes(i, j);
	  }
	if (LENGTH > 1)
	  {
	     j = LENGTH - 1;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	     HorFuseVerRes(i, j);
	     VerFuse(i, j);
	     VerFuseHorRes(i, j);
	  }
	if (LENGTH >= 1)
	  {
	     j = LENGTH;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     HorFuse(i, j);
	  }
     }
   if (WIDTH >= 1)
     {
	i = WIDTH;
	for (j = 1; j < LENGTH - 1; j++)
	  {
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     VerFuse(i, j);
	     VerFuseVerRes(i, j);
	  }
	if (LENGTH > 1)
	  {
	     j = LENGTH - 1;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	     VerFuse(i, j);
	  }
	if (LENGTH >= 1)
	  {
	     j = LENGTH;
	     Source(i, j, input[(j-1) * WIDTH + (i-1)]);
	  }
     }
   f = fopen("footer.cir", "r");
   while ((c = getc(f)) != EOF)
     {
	putchar(c);
     }
   fclose(f);
   
   return 0;
}


