#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int width, length;

#define PIXELSIZE 1
/*
#define WIDTH 8
#define LENGTH 4
*/
#define WIDTH width
#define LENGTH length

void ReadVoltages(const char *filename, float *voltages, int width, int length)
{
   FILE *f = fopen(filename, "r");
   int a, position, i, j;
   char guff;
   float next;
   
   for (a = 0; a < width * length; a++)
     {
	fscanf(f, " V( %c%d ) %f\n", &guff, &position, &next);
	i = position / 100;
	j = position % 100;
	voltages[(i - 1) + (j - 1) * width] = next;
     }
   fclose(f);
}

void ReadCurrents(const char *filename, float *currents, int width, int length)
{
   FILE *f = fopen(filename, "r");
   int a, position, i, j;
   char guff;
   float next;
   for (a = 0; a < width * length; a++)
     {
	fscanf(f, "\tvequalise:x%c%d#branch %f\n", &guff, &position, &next);
	i = position / 100;
	j = position % 100;
	currents[(i-1) + (j-1) * width] = next;
     }
   fclose(f);
}

void PrintHeader(FILE *f, const char *hfile, int width, int length)
{
   FILE *header;
   char c;

   fprintf(f, "%%!PS-Adobe-2.0\n");
   fprintf(f, "%%%%BoundingBox: 0 0 %d %d\n\n",
	   PIXELSIZE * width, PIXELSIZE * length);
   header = fopen(hfile, "r");
   while ((c = getc(header)) != EOF)
     putc(c, f);
   fclose(header);
}

void PrintVerticalEdges(int *veredges, int width, int length, FILE *f)
{
   int i, j;

   for (i = 0; i < width; i++)
     {
	for (j = 0; j < length; j++)
	  {
	     if (veredges[i + j * width] == 1)
	       {
		  fprintf(f, "%d %d %d %d dotline\n", i * PIXELSIZE,
			  (j + 1) * PIXELSIZE, (i + 1) * PIXELSIZE, (j + 1) * PIXELSIZE);
	       }
	  }
     }
}

void PrintHorizontalEdges(int *horedges, int width, int length, FILE *f)
{
   int i, j;

   for (i = 0; i < width; i++)
     {
	for (j = 0; j < length; j++)
	  {
	     if (horedges[i + j * width] == 1)
	       {
		  fprintf(f, "%d %d %d %d dotline\n", (i + 1) * PIXELSIZE,
			  j * PIXELSIZE, (i + 1) * PIXELSIZE, (j + 1) * PIXELSIZE);
	       }
	  }
     }
}

void PrintPixels(float *intensities, int width, int length, FILE *f)
{
   int i, j;

   for (i = 0; i < width; i++)
     {
	for (j = 0; j < length; j++)
	  fprintf(f, "%3d %3d %.3f pixel\n", i * PIXELSIZE, j * PIXELSIZE, intensities[j * width + i]);
     }
}

void PrintOutput(char *input, char *output, char *hedge, char *vedge,
		 int width, int length)
{
   FILE *out = fopen(output, "w");
   float *rawintensities = (float *)malloc(sizeof(float) * width * length);
   float *intensities = (float *)malloc(sizeof(float) * width * length);
   float *rawhedges = (float *)malloc(sizeof(float) * (width - 1) * length);
   float *rawhres = (float *)malloc(sizeof(float) * (width - 1) * length);
   int *hedges = (int *)malloc(sizeof(int) * (width - 1) * length);
   float *rawvedges = (float *)malloc(sizeof(float) * width * (length - 1));
   float *rawvres = (float *)malloc(sizeof(float) * (width - 1) * length);
   int *vedges = (int *)malloc(sizeof(int) * width * (length - 1));
   int i, j;
   float vdiff;

   ReadVoltages(input, rawintensities, width, length);
   for (i = 0; i < width; i++)
     {
	for (j = 0; j < length; j++)
	  intensities[i + j * width] = rawintensities[i + j * width] - 2.0;
     }
   PrintHeader(out, "header.ps", width, length);
   PrintPixels(intensities, width, length, out);

   if (hedge != NULL)
     {
	ReadCurrents(hedge, rawhedges, width - 1, length);
	for (i = 0; i < width - 1; i++)
	  {
	     for (j = 0; j < length; j++)
	       {
		  vdiff = rawintensities[i + j * width] - rawintensities[i + 1 + j * width];
		  rawhres[i + j * (width - 1)] = fabs(vdiff / rawhedges[i + j * (width - 1)]);
		  hedges[i + j * (width - 1)] = (rawhres[i + j * (width - 1)] > 1e6);
	       }
	  }
	PrintHorizontalEdges(hedges, width - 1, length, out);
     }

   if (vedge != NULL)
     {
	ReadCurrents(vedge, rawvedges, width, length - 1);
	for (i = 0; i < width; i++)
	  {
	     for (j = 0; j < length - 1; j++)
	       {
		  vdiff = rawintensities[i + j * width] - rawintensities[i + width + j * width];
		  rawvres[i + j * width] = fabs(vdiff / rawvedges[i + j * width]);
		  vedges[i + j * width] = (rawvres[i + j * width] > 1e6);
	       }
	  }
	PrintVerticalEdges(vedges, width, length - 1, out);
     }

   fclose(out);
}

int main()
{
   FILE *f;
   
   f = fopen("width.out", "r");
   fscanf(f, "*NETWORKWIDTH %d\n", &width);
   f = fopen("length.out", "r");
   fscanf(f, "*NETWORKLENGTH %d\n", &length);
   PrintOutput("signal.out", "signal.ps", NULL, NULL, WIDTH, LENGTH);
   PrintOutput("noisy.out", "noisy.ps", NULL, NULL, WIDTH, LENGTH);
   PrintOutput("output.out", "out.ps", NULL, NULL, WIDTH, LENGTH);
   PrintOutput("output.out", "line.ps", "horedges.out", "veredges.out", WIDTH, LENGTH);
   return 0;
}

